@@ -1,5 +1,5 @@
-#!/usr/bin/env python
-
+# -*- coding: utf-8 -*-
+#
 # Copyright 2012 GRNET S.A. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or
@@ -33,15 +33,22 @@
 # interpreted as representing official policies, either expressed
 # or implied, of GRNET S.A.
 
+"""This module implements the "wizard" mode of the dialog-based version of
+snf-image-creator.
+"""
+
 import time
 import StringIO
+import json
 
 from image_creator.kamaki_wrapper import Kamaki, ClientError
 from image_creator.util import MD5, FatalError
 from image_creator.output.cli import OutputWthProgress
-from image_creator.dialog_util import extract_image, update_background_title
+from image_creator.dialog_util import extract_image, update_background_title, \
+    add_cloud, edit_cloud
 
 PAGE_WIDTH = 70
+PAGE_HEIGHT = 10
 
 
 class WizardExit(Exception):
@@ -49,8 +56,8 @@ class WizardExit(Exception):
     pass
 
 
-class WizardInvalidData(Exception):
-    """Exception triggered when the user provided data are invalid"""
+class WizardReloadPage(Exception):
+    """Exception that reloads the last WizardPage"""
     pass
 
 
@@ -76,20 +83,22 @@ class Wizard:
         idx = 0
         while True:
             try:
-                idx += self.pages[idx].run(self.session, idx, len(self.pages))
+                total = len(self.pages)
+                title = "(%d/%d) %s" % (idx + 1, total, self.pages[idx].title)
+                idx += self.pages[idx].run(self.session, title)
             except WizardExit:
                 return False
-            except WizardInvalidData:
+            except WizardReloadPage:
                 continue
 
             if idx >= len(self.pages):
-                msg = "All necessary information has been gathered:\n\n"
+                text = "All necessary information has been gathered:\n\n"
                 for page in self.pages:
-                    msg += " * %s\n" % page.info
-                msg += "\nContinue with the image creation process?"
+                    text += " * %s\n" % page.info
+                text += "\nContinue with the image creation process?"
 
                 ret = self.d.yesno(
-                    msg, width=PAGE_WIDTH, height=8 + len(self.pages),
+                    text, width=PAGE_WIDTH, height=8 + len(self.pages),
                     ok_label="Yes", cancel="Back", extra_button=1,
                     extra_label="Quit", title="Confirmation")
 
@@ -109,14 +118,26 @@ class WizardPage(object):
     NEXT = 1
     PREV = -1
 
-    def __init__(self, **kargs):
+    def __init__(self, name, display_name, text, **kargs):
+        self.name = name
+        self.display_name = display_name
+        self.text = text
+
+        self.title = kargs['title'] if 'title' in kargs else ""
+        self.default = kargs['default'] if 'default' in kargs else ""
+        self.extra = kargs['extra'] if 'extra' in kargs else None
+        self.extra_label = \
+            kargs['extra_label'] if 'extra_label' in kargs else 'Extra'
+
+        self.info = "%s: <none>" % self.display_name
+
         validate = kargs['validate'] if 'validate' in kargs else lambda x: x
         setattr(self, "validate", validate)
 
         display = kargs['display'] if 'display' in kargs else lambda x: x
         setattr(self, "display", display)
 
-    def run(self, session, index, total):
+    def run(self, session, title):
         """Display this wizard page
 
         This function is used by the wizard program when accessing a page.
@@ -124,123 +145,175 @@ class WizardPage(object):
         raise NotImplementedError
 
 
-class WizardRadioListPage(WizardPage):
-    """Represent a Radio List in a wizard"""
-    def __init__(self, name, printable, message, choices, **kargs):
-        super(WizardRadioListPage, self).__init__(**kargs)
-        self.name = name
-        self.printable = printable
-        self.message = message
-        self.choices = choices
-        self.title = kargs['title'] if 'title' in kargs else ''
-        self.default = kargs['default'] if 'default' in kargs else ""
+class WizardPageWthChoices(WizardPage):
+    """Represents a Wizard Page that allows the user to select something from
+    a list of choices.
 
-    def run(self, session, index, total):
+    The available choices are created by a function passed to the class through
+    the choices variable. If the choices function returns an empty list, a
+    fallback funtion is executed if available.
+    """
+    def __init__(self, name, display_name, text, choices, **kargs):
+        super(WizardPageWthChoices, self).__init__(name, display_name, text,
+                                                   **kargs)
+        self.choices = choices
+        self.fallback = kargs['fallback'] if 'fallback' in kargs else None
+
+
+class WizardRadioListPage(WizardPageWthChoices):
+    """Represent a Radio List in a wizard"""
+
+    def run(self, session, title):
         d = session['dialog']
         w = session['wizard']
 
         choices = []
-        for i in range(len(self.choices)):
-            default = 1 if self.choices[i][0] == self.default else 0
-            choices.append((self.choices[i][0], self.choices[i][1], default))
+        for choice in self.choices():
+            default = 1 if choice[0] == self.default else 0
+            choices.append((choice[0], choice[1], default))
 
         (code, answer) = d.radiolist(
-            self.message, height=10, width=PAGE_WIDTH, ok_label="Next",
-            cancel="Back", choices=choices,
-            title="(%d/%d) %s" % (index + 1, total, self.title))
+            self.text, width=PAGE_WIDTH, ok_label="Next", cancel="Back",
+            choices=choices, height=PAGE_HEIGHT, title=title)
 
         if code in (d.DIALOG_CANCEL, d.DIALOG_ESC):
             return self.PREV
 
         w[self.name] = self.validate(answer)
         self.default = answer
-        self.info = "%s: %s" % (self.printable, self.display(w[self.name]))
+        self.info = "%s: %s" % (self.display_name, self.display(w[self.name]))
 
         return self.NEXT
 
 
 class WizardInputPage(WizardPage):
     """Represents an input field in a wizard"""
-    def __init__(self, name, printable, message, **kargs):
-        super(WizardInputPage, self).__init__(**kargs)
-        self.name = name
-        self.printable = printable
-        self.message = message
-        self.info = "%s: <none>" % self.printable
-        self.title = kargs['title'] if 'title' in kargs else ''
-        self.init = kargs['init'] if 'init' in kargs else ''
 
-    def run(self, session, index, total):
+    def run(self, session, title):
         d = session['dialog']
         w = session['wizard']
 
         (code, answer) = d.inputbox(
-            self.message, init=self.init, width=PAGE_WIDTH, ok_label="Next",
-            cancel="Back", title="(%d/%d) %s" % (index + 1, total, self.title))
+            self.text, init=self.default, width=PAGE_WIDTH, ok_label="Next",
+            cancel="Back", height=PAGE_HEIGHT, title=title)
 
         if code in (d.DIALOG_CANCEL, d.DIALOG_ESC):
             return self.PREV
 
         value = answer.strip()
-        self.init = value
+        self.default = value
         w[self.name] = self.validate(value)
-        self.info = "%s: %s" % (self.printable, self.display(w[self.name]))
+        self.info = "%s: %s" % (self.display_name, self.display(w[self.name]))
+
+        return self.NEXT
+
+
+class WizardMenuPage(WizardPageWthChoices):
+    """Represents a menu dialog with available choices in a wizard"""
+
+    def run(self, session, title):
+        d = session['dialog']
+        w = session['wizard']
+
+        extra_button = 1 if self.extra else 0
+
+        choices = self.choices()
+
+        if len(choices) == 0:
+            assert self.fallback, "Zero choices and no fallback"
+            if self.fallback():
+                raise WizardReloadPage
+            else:
+                return self.PREV
+
+        default_item = self.default if self.default else choices[0][0]
+
+        (code, choice) = d.menu(
+            self.text, width=PAGE_WIDTH, ok_label="Next", cancel="Back",
+            title=title, choices=choices, height=PAGE_HEIGHT,
+            default_item=default_item, extra_label=self.extra_label,
+            extra_button=extra_button)
+
+        if code in (d.DIALOG_CANCEL, d.DIALOG_ESC):
+            return self.PREV
+        elif code == d.DIALOG_EXTRA:
+            self.extra()
+            raise WizardReloadPage
+
+        self.default = choice
+        w[self.name] = self.validate(choice)
+        self.info = "%s: %s" % (self.display_name, self.display(w[self.name]))
 
         return self.NEXT
 
 
 def start_wizard(session):
     """Run the image creation wizard"""
-    init_token = Kamaki.get_token()
-    if init_token is None:
-        init_token = ""
 
     distro = session['image'].distro
     ostype = session['image'].ostype
+
+    def cloud_choices():
+        choices = []
+        for (name, cloud) in Kamaki.get_clouds().items():
+            descr = cloud['description'] if 'description' in cloud else ''
+            choices.append((name, descr))
+
+        return choices
+
+    def cloud_add():
+        return add_cloud(session)
+
+    def cloud_none_available():
+        if not session['dialog'].yesno(
+                "No available clouds found. Would you like to add one now?",
+                width=PAGE_WIDTH, defaultno=0):
+            return add_cloud(session)
+        return False
+
+    def cloud_validate(cloud):
+        if not Kamaki.get_account(cloud):
+            if not session['dialog'].yesno(
+                    "The cloud you have selected is not valid! Would you "
+                    "like to edit it now?", width=PAGE_WIDTH, defaultno=0):
+                if edit_cloud(session, cloud):
+                    return cloud
+
+            raise WizardInvalidData
+
+        return cloud
+
+    cloud = WizardMenuPage(
+        "Cloud", "Cloud",
+        "Please select a cloud account or press <Add> to add a new one:",
+        choices=cloud_choices, extra_label="Add", extra=cloud_add,
+        title="Clouds", validate=cloud_validate, fallback=cloud_none_available)
+
     name = WizardInputPage(
         "ImageName", "Image Name", "Please provide a name for the image:",
-        title="Image Name", init=ostype if distro == "unknown" else distro)
+        title="Image Name", default=ostype if distro == "unknown" else distro)
 
     descr = WizardInputPage(
         "ImageDescription", "Image Description",
         "Please provide a description for the image:",
-        title="Image Description", init=session['metadata']['DESCRIPTION'] if
-        'DESCRIPTION' in session['metadata'] else '')
+        title="Image Description", default=session['metadata']['DESCRIPTION']
+        if 'DESCRIPTION' in session['metadata'] else '')
+
+    def registration_choices():
+        return [("Private", "Image is accessible only by this user"),
+                ("Public", "Everyone can create VMs from this image")]
 
     registration = WizardRadioListPage(
         "ImageRegistration", "Registration Type",
-        "Please provide a registration type:",
-        [("Private", "Image is accessible only by this user"),
-         ("Public", "Everyone can create VMs from this image")],
+        "Please provide a registration type:", registration_choices,
         title="Registration Type", default="Private")
 
-    def validate_account(token):
-        """Check if a token is valid"""
-        d = session['dialog']
-
-        if len(token) == 0:
-            d.msgbox("The token cannot be empty", width=PAGE_WIDTH)
-            raise WizardInvalidData
-
-        account = Kamaki.get_account(token)
-        if account is None:
-            d.msgbox("The token you provided in not valid!", width=PAGE_WIDTH)
-            raise WizardInvalidData
-
-        return account
-
-    account = WizardInputPage(
-        "Account", "Account",
-        "Please provide your ~okeanos authentication token:",
-        title="~okeanos account", init=init_token, validate=validate_account,
-        display=lambda account: account['username'])
-
     w = Wizard(session)
 
+    w.add_page(cloud)
     w.add_page(name)
     w.add_page(descr)
     w.add_page(registration)
-    w.add_page(account)
 
     if w.run():
         create_image(session)
@@ -256,9 +329,6 @@ def create_image(session):
     image = session['image']
     wizard = session['wizard']
 
-    # Save Kamaki credentials
-    Kamaki.save_token(wizard['Account']['auth_token'])
-
     with_progress = OutputWthProgress(True)
     out = image.out
     out.add(with_progress)
@@ -266,16 +336,8 @@ def create_image(session):
         out.clear()
 
         #Sysprep
-        image.mount(False)
-        err_msg = "Unable to execute the system preparation tasks."
-        if not image.mounted:
-            raise FatalError("%s Couldn't mount the media." % err_msg)
-        elif image.mounted_ro:
-            raise FatalError("%s Couldn't mount the media read-write."
-                             % err_msg)
         image.os.do_sysprep()
         metadata = image.os.meta
-        image.umount()
 
         #Shrink
         size = image.shrink()
@@ -289,29 +351,22 @@ def create_image(session):
         md5 = MD5(out)
         session['checksum'] = md5.compute(image.device, size)
 
-        #Metadata
-        metastring = '\n'.join(
-            ['%s=%s' % (key, value) for (key, value) in metadata.items()])
-        metastring += '\n'
-
         out.output()
         try:
-            out.output("Uploading image to pithos:")
-            kamaki = Kamaki(wizard['Account'], out)
+            out.output("Uploading image to the cloud:")
+            account = Kamaki.get_account(wizard['Cloud'])
+            assert account, "Cloud: %s is not valid" % wizard['Cloud']
+            kamaki = Kamaki(account, out)
 
             name = "%s-%s.diskdump" % (wizard['ImageName'],
                                        time.strftime("%Y%m%d%H%M"))
             pithos_file = ""
             with open(image.device, 'rb') as f:
                 pithos_file = kamaki.upload(f, size, name,
-                                            "(1/4)  Calculating block hashes",
-                                            "(2/4)  Uploading missing blocks")
+                                            "(1/3)  Calculating block hashes",
+                                            "(2/3)  Uploading missing blocks")
 
-            out.output("(3/4)  Uploading metadata file ...", False)
-            kamaki.upload(StringIO.StringIO(metastring), size=len(metastring),
-                          remote_path="%s.%s" % (name, 'meta'))
-            out.success('done')
-            out.output("(4/4)  Uploading md5sum file ...", False)
+            out.output("(3/3)  Uploading md5sum file ...", False)
             md5sumstr = '%s %s\n' % (session['checksum'], name)
             kamaki.upload(StringIO.StringIO(md5sumstr), size=len(md5sumstr),
                           remote_path="%s.%s" % (name, 'md5sum'))
@@ -320,11 +375,17 @@ def create_image(session):
 
             is_public = True if wizard['ImageRegistration'] == "Public" else \
                 False
-            out.output('Registering %s image with ~okeanos ...' %
+            out.output('Registering %s image with the cloud ...' %
                        wizard['ImageRegistration'].lower(), False)
-            kamaki.register(wizard['ImageName'], pithos_file, metadata,
-                            is_public)
+            result = kamaki.register(wizard['ImageName'], pithos_file,
+                                     metadata, is_public)
             out.success('done')
+            out.output("Uploading metadata file ...", False)
+            metastring = unicode(json.dumps(result, ensure_ascii=False))
+            kamaki.upload(StringIO.StringIO(metastring), size=len(metastring),
+                          remote_path="%s.%s" % (name, 'meta'))
+            out.success('done')
+
             if is_public:
                 out.output("Sharing md5sum file ...", False)
                 kamaki.share("%s.md5sum" % name)
@@ -336,14 +397,17 @@ def create_image(session):
             out.output()
 
         except ClientError as e:
-            raise FatalError("Pithos client: %d %s" % (e.status, e.message))
+            raise FatalError("Storage service client: %d %s" %
+                             (e.status, e.message))
     finally:
         out.remove(with_progress)
 
-    msg = "The %s image was successfully uploaded and registered with " \
-          "~okeanos. Would you like to keep a local copy of the image?" \
-          % wizard['ImageRegistration'].lower()
-    if not d.yesno(msg, width=PAGE_WIDTH):
+    text = "The %s image was successfully uploaded to the storage service " \
+           "and registered with the compute service of %s. Would you like " \
+           "to keep a local copy?" % \
+           (wizard['Cloud'], wizard['ImageRegistration'].lower())
+
+    if not d.yesno(text, width=PAGE_WIDTH):
         extract_image(session)
 
 # vim: set sta sts=4 shiftwidth=4 sw=4 et ai :
